// Bound on polynomial solutions of the Risch DE -- nonlinear case

// given a, b, c in k[t], degree(d(t), [t])>1, a<>0, return n in Z_
// such that degree(q) <= n for any solution q in k[t] of 
// a*diff(q,t)+b*q=c.

// Follows Bronstein, Symbolic Integration I, p. 201

intlib::algebraic::rde::boundDegreeNonlinear :=
proc(a, b, c, ts, diffs, algs)
  local t, dt, da, db, dc, delta, lambda, m, n;
begin
  t := ts[-1];
  dt := poly(diffs[-1], [t]);
  assert(degree(dt)>1);
  da := degree(a);
  db := degree(b);
  dc := degree(c);
  delta := degree(dt);
  lambda := lcoeff(dt);
  n := max(0, dc-max(da+delta-1, db));
  if db=da+delta-1 then // possible cancellation
    m := normal(-lcoeff(b)/(lambda*lcoeff(a)));
    if testtype(m, Type::Integer) then
      n := max(0, m, dc-db);
    end_if;
  end_if;
  return(n);
end:
