ellipticE::series:=proc(f, z, n, dir, opt)
local z0, x, s, t, k, ph, p, q, r, pp, qq, i, j;
begin
  // this is only about the complete integral
  if args(0)>4 then
    return(Series::unknown(ellipticE(args(1..2)), args(3..5)));
  end_if;
  
  x:=genident();

  t:=Series::series(f, z, n, dir, opt);

  // pochhammer symbol:
  ph:=proc(x,n)
  local y,k;
  begin
    y:=1;
    for k from 1 to n do
      y:=y*(x+k-1)
    end;
    return(y);
  end;

  k:=Series::Puiseux::ldegree(t);

  if k>0 then // expansion around 0
    s:=Series::Puiseux::create(1, 0, n,
      [PI/2*ph(-1/2,k)*ph(1/2,k)/(k!)^2 $ k=0..(n-1)], z, 0, dir);

    return(s @ t);
  elif k<0 then // expansion around infinity
    s:=Series::Puiseux::create(1, -1, n,
      [-1/sqrt(-1/z), ph(-1/2,j+1)^2/j!/(j+1)!*(ln(-1/z)+2*psi(j+1)-2*psi(1/2-j)+1/(j+1))/sqrt(-1/z) $ j=0..(n-2)], z, 0, Undirected);

    t:=Series::series(1/f, z, n, Undirected, opt);
    return(s @ t);
  else // k=0
    z0:=lcoeff(t);
    if z0=1 then // expansion around 1
      s:=Series::Puiseux::create(1, 0, n,
        [1, (-1)^j*ph(1/2,j)*ph(3/2,j)/j!/(j+1)!*(ln(1-z)-2*psi(j+1)+2*psi(j+1/2)+1/(2*j^2+3*j+1))/4 $ j=0..(n-2)], z, 1, dir);

      return(s @ t);
    else // expansion around any other point
      if Re(z0)>1 and Im(z0)=0 then
        s:=[ ellipticE(z0) + (signIm(z)+1)*I*(ellipticCK(z0)-ellipticCE(z0))];
      else
        s:=[ ellipticE(z0) ];
      end;
      
      p:=-1;
      q:=1;
      r:=2*z0;
      for i from 1 to n do
        // Loop invariant:
        // subs(diff(ellipticE(x), x$i), x=z0) = subs(ellipticK(x)*p+ellipticE(x)*q, x=z0)/r
        
        pp:=subs(p, x=z0);
        qq:=subs(q, x=z0);
        
        if Re(z0)>1 and Im(z0)=0 then // z0 is on the branch cut
          s:=append(s, (ellipticK(z0)*pp+ellipticE(z0)*qq+(signIm(z)+1)*I*(ellipticCK(z0)*(pp+qq)-ellipticCE(z0)*qq))/r);
        else
          s:=append(s, (ellipticK(z0)*pp+ellipticE(z0)*qq)/r);
        end;
        
        [p, q]:=expand(
          [(2*i+x-4*i*x+1)*p+(1-x)*q+2*x*(x-1)*diff(p, x),
           (2*i+3*x-4*i*x-1)*q-p+2*x*(x-1)*diff(q, x)]);
        r:=r*2*z0*(z0-1)*(i+1);
      end;
      s:=Series::Puiseux::create(1, 0, n, s, z, z0, dir);
      return(s @ t);
    end;
  end;
end_proc:


