// arccot::simplify

arccot::simplify:=
proc(a: "arccot", options = simplify::defaultOptions: DOM_TABLE)
  local /*reciprocalIsSimpler: DOM_PROC,*/ x, mtch;
begin

  x:= op(a, 1);  // a = arccot(x)

  if type(x) = "cot" then
    x:= op(x, 1);
    if options[IgnoreAnalyticConstraints] or
       is( (              Re(x)> -PI/2 and Re(x)< PI/2) or
           (Im(x)>= 0 and Re(x)> -PI/2 and Re(x)<=PI/2) or
           (Im(x)<= 0 and Re(x)>=-PI/2 and Re(x)< PI/2) or
           (Im(x) = 0 and Re(x)>=-PI/2 and Re(x)<=PI/2)
          ) = TRUE then
         return(x)
    end_if;
  elif type(-x) = "cot" then // -cot(y) = cot(-y)
    x:= op(-x, 1);
    if options[IgnoreAnalyticConstraints] or
       is( (              Re(x)> -PI/2 and Re(x)< PI/2) or
           (Im(x)>= 0 and Re(x)> -PI/2 and Re(x)<=PI/2) or
           (Im(x)<= 0 and Re(x)>=-PI/2 and Re(x)< PI/2) or
           (Im(x) = 0 and Re(x)>=-PI/2 and Re(x)<=PI/2)
          ) = TRUE then
         return(-x)
    end_if;
  elif type(x/I) = "coth" then // x = I*coth(y) = cot(-y*I)
    x:= op(x/I, 1);
    if options[IgnoreAnalyticConstraints] or
       is( (              Im(x)> -PI/2 and Im(x)< PI/2) or
           (Re(x)>= 0 and Im(x)>=-PI/2 and Im(x)< PI/2) or
           (Re(x)<= 0 and Im(x)> -PI/2 and Im(x)<=PI/2) or
           (Re(x) = 0 and Im(x)>=-PI/2 and Im(x)<=PI/2)
          ) = TRUE then
         return(-x*I)
    end_if;
  elif type(-x/I) = "coth" then // x = -I*coth(y) = cot(y*I)
    x:= op(-x/I, 1);
    if options[IgnoreAnalyticConstraints] or
       is( (              Im(x)> -PI/2 and Im(x)< PI/2) or
           (Re(x)>= 0 and Im(x)>=-PI/2 and Im(x)< PI/2) or
           (Re(x)<= 0 and Im(x)> -PI/2 and Im(x)<=PI/2) or
           (Re(x) = 0 and Im(x)>=-PI/2 and Im(x)<=PI/2)
          ) = TRUE then
         return(x*I)
    end_if;
  elif type(1/x) = "tan" then //  x = 1/tan(y) = cot(y)
    x:= op(1/x, 1);
    if options[IgnoreAnalyticConstraints] or
       is( (              Re(x)> -PI/2 and Re(x)< PI/2) or
           (Im(x)>= 0 and Re(x)> -PI/2 and Re(x)<=PI/2) or
           (Im(x)<= 0 and Re(x)>=-PI/2 and Re(x)< PI/2) or
           (Im(x) = 0 and Re(x)>=-PI/2 and Re(x)<=PI/2)
          ) = TRUE then
         return(x)
    end_if
  elif type(-1/x) = "tan"  then // x =  -1/tan(y) = cot(-y)
    x:= op(-1/x, 1);
    if options[IgnoreAnalyticConstraints] or
       is( (              Re(x)> -PI/2 and Re(x)< PI/2) or
           (Im(x)>= 0 and Re(x)> -PI/2 and Re(x)<=PI/2) or
           (Im(x)<= 0 and Re(x)>=-PI/2 and Re(x)< PI/2) or
           (Im(x) = 0 and Re(x)>=-PI/2 and Re(x)<=PI/2)
          ) = TRUE then
         return(-x)
    end_if
  elif type(I/x) = "tanh" then // x = I/tanh(y) = cot(-y*I)
    x:= op(I/x, 1);
    if options[IgnoreAnalyticConstraints] or
       is( (              Im(x)> -PI/2 and Im(x)< PI/2) or
           (Re(x)>= 0 and Im(x)>=-PI/2 and Im(x)< PI/2) or
           (Re(x)<= 0 and Im(x)> -PI/2 and Im(x)<=PI/2) or
           (Re(x) = 0 and Im(x)>=-PI/2 and Im(x)<=PI/2)
          ) = TRUE then
         return(-x*I)
    end_if
  elif type(-I/x) = "tanh" then // x = -I/tanh(y) = cot(y*I)
    x:= op(-I/x, 1);
    if options[IgnoreAnalyticConstraints] or
       is( (              Im(x)> -PI/2 and Im(x)< PI/2) or
           (Re(x)>= 0 and Im(x)>=-PI/2 and Im(x)< PI/2) or
           (Re(x)<= 0 and Im(x)> -PI/2 and Im(x)<=PI/2) or
           (Re(x) = 0 and Im(x)>=-PI/2 and Im(x)<=PI/2)
          ) = TRUE then
         return(x*I)
    end_if
  end_if:

  //====================================================
  // We still wish to implement simplifications based on
  //
  //  arccot(z) + arccot(1/z) = PI/2*signIm(I*z) * s(z)
  //
  // where s(z) = 1 for nearly all z in C_ apart from
  // the intervals z/I in (-1, 0) union (0, 1) along the
  // real line, where s(z) = -1.
  //====================================================
  // Start over again
  x:= op(a, 1);

  if (mtch:= match(x, cos(`#X`)/sin(`#X`))) <> FAIL then
     return(simplify(arccot(cot(op(mtch, [1, 2]))), options))
  end_if;

/*
  //==========================================
  reciprocalIsSimpler:= proc(x)
    local y;
  begin
    case domtype(x)
      of DOM_INT do return(FALSE);
      of DOM_RAT do return(bool(abs(x) < 1));
    end_case;
    y:= 1/x;
    case specfunc::sign(length(x) - length(y))
      of  1 do return(TRUE);
      of -1 do return(FALSE);
      of 0 do return(sysorder(y, x));
    end_case;
  end_proc;
  //==========================================

  case sign(Re(x))
    of 1 do
    of 0 do
      if reciprocalIsSimpler(x) then
        return(-arccot(1/x) + 1/2*PI)
      else
        break
      end_if;
    of -1 do
      if reciprocalIsSimpler(1/x, x) then
         return(-arccot(1/x) - 1/2*PI)
      else
        break
      end_if
  end_case;
*/
  
  return(a)
end_proc:
